<?php

namespace Drp\ProjectHelpers\Tests\Concerns;

use Drp\ProjectHelpers\Tests\Fakes\FakeFileSystem;
use Illuminate\Http\UploadedFile;
use League\Flysystem\Filesystem;
use Mockery;
use Symfony\Component\DomCrawler\Form;

/**
 * Class InteractsWithFilesystem
 *
 * @package Drp\ProjectHelpers
 */
trait InteractsWithFilesystem
{
    /**
     * @var
     */
    protected $filesystem;

    /**
     * @return \League\Flysystem\Filesystem
     */
    public function setVirtualFileStructure()
    {
        $adapter = new FakeFileSystem();
        $this->filesystem = new Filesystem($adapter);

        app('filesystem')->extend('vfs', function () {
            return $this->filesystem;
        });

        config([
            'filesystems.disks.virtual.driver' => 'vfs',
            'filesystems.default' => 'virtual',
        ]);
        return $this->filesystem;
    }

    /**
     * @param $path
     * @return $this
     */
    protected function assertFileInFileSystem($path)
    {
        $this->assertTrue($this->filesystem->has($path), 'Cannot find file in directory');
        return $this;
    }

    /**
     * @param $file
     * @param $hashName
     * @return Mockery\MockInterface
     */
    protected function createFileUploadMock($file, $hashName)
    {
        $file = Mockery::mock(UploadedFile::class . '[hashName]', [$file, null, null, null, null, true]);
        $file->shouldReceive('hashName')
            ->atMost(1)
            ->andReturn($hashName);

        return $file;
    }

    /**
     * Convert the given uploads to UploadedFile instances.
     *
     * @param  \Symfony\Component\DomCrawler\Form $form
     * @param  array $uploads
     * @return array
     */
    protected function convertUploadsForTesting(Form $form, array $uploads)
    {
        $files = $form->getFiles();
        $names = array_keys($files);

        $files = array_map(function (array $file, $name) use ($uploads) {
            return isset($uploads[$name]) ? $uploads[$name] : null;
        }, $files, $names);

        $arr = [];
        collect(array_combine($names, $files))
            ->each(function ($file, $key) use (&$arr) {
                preg_match('/([a-z_]*)([[]\d[]])/', $key, $matches);
                $key = (count($matches) > 0) ? $matches[1] : $key;
                $arr[$key] = $file;
            });

        return $arr;
    }
}
