<?php

namespace Drp\ProjectHelpers\Test\Unit;

use Drp\ProjectHelpers\Test\TestCase;
use Drp\ProjectHelpers\Tests\Fakes\FakeFileSystem;
use League\Flysystem\Config;

class FakeFileSystemTest extends TestCase
{

    /**
     * @test
     */
    public function stores_a_file_in_the_filesystem()
    {
        $files = new FakeFileSystem();
        $files->write('/path/info', 'FILE', new Config());

        $this->assertCount(1, $files->files);
        $this->assertArrayHasKey('/path/info', $files->files);
        $this->assertContains('FILE', $files->files);

        $files->update('/path/info', 'FILE_UPDATED', new Config());

        $this->assertCount(1, $files->files);
        $this->assertContains('FILE_UPDATED', $files->files);
    }

    /**
     * @test
     */
    public function renames_a_file_in_the_filesystem()
    {
        $files = new FakeFileSystem();
        $files->write('/path/info', 'FILE', new Config());
        $files->rename('/path/info', '/path/info2', new Config());

        $this->assertCount(1, $files->files);
        $this->assertArrayHasKey('/path/info2', $files->files);
        $this->assertContains('FILE', $files->files['/path/info2']);
        $this->assertArrayNotHasKey('/path/info', $files->files);
    }

    /**
     * @test
     */
    public function copies_a_file_in_the_filesystem()
    {
        $files = new FakeFileSystem();
        $files->write('/path/info', 'FILE', new Config());
        $files->copy('/path/info', '/path/info2', new Config());

        $this->assertCount(2, $files->files);
        $this->assertArrayHasKey('/path/info2', $files->files);
        $this->assertContains('FILE', $files->files['/path/info2']);
        $this->assertArrayHasKey('/path/info', $files->files);
        $this->assertContains('FILE', $files->files['/path/info']);
    }

    /**
     * @test
     */
    public function deletes_a_file_in_the_filesystem()
    {
        $files = new FakeFileSystem();
        $files->write('/path/info', 'FILE', new Config());
        $files->delete('/path/info');

        $this->assertEmpty($files->files);
    }

    /**
     * @test
     */
    public function returns_if_file_exists_in_filesystem()
    {
        $files = new FakeFileSystem();
        $files->write('/path/info', 'FILE', new Config());

        $this->assertTrue($files->has('/path/info'));
    }

    /**
     * @test
     */
    public function reads_a_file_in_filesystem()
    {
        $files = new FakeFileSystem();
        $files->write('/path/info', 'FILE', new Config());

        $this->assertEquals('FILE', $files->read('/path/info'));
    }

}